@@ -19,25 +19,54 @@ class Parser(object):
                        disjunct_cost=10000,
                        max_sentence_length=170):
         locale.setlocale(locale.LC_ALL,"en_US.UTF8")
-
-        self.parse_options = ParseOptions(verbosity=verbosity,
-                                          linkage_limit=linkage_limit,
-                                          min_null_count=min_null_count,
-                                          max_null_count=max_null_count,
-                                          null_block=null_block,
-                                          islands_ok=islands_ok,
-                                          short_length=short_length,
-                                          display_walls=display_walls,
-                                          allow_null=allow_null,
-                                          screen_width=screen_width,
-                                          max_parse_time=max_parse_time,
-                                          max_memory=max_memory,
-                                          disjunct_cost=disjunct_cost,
-                                          max_sentence_length=max_sentence_length)
+        
+        #We need to retain all these options cause we will
+        #recreate the parse_options if the parser starts failing
+        self.verbosity = verbosity
+        self.linkage_limit = linkage_limit
+        self.min_null_count = min_null_count
+        self.max_null_count = max_null_count
+        self.null_block = null_block
+        self.islands_ok = islands_ok
+        self.short_length = short_length
+        self.all_short_connectors = all_short_connectors
+        self.display_walls = display_walls
+        self.screen_width = screen_width
+        self.max_parse_time = max_parse_time
+        self.max_memory = max_memory
+        self.disjunct_cost = disjunct_cost
+        self.max_sentence_length = max_sentence_length
+        self.allow_null = allow_null
+        
+        self._create_parse_options()
         self.dictionary = Dictionary("en")
+
+    def _create_parse_options(self):
+        self.parse_options = ParseOptions(verbosity=self.verbosity,
+                                          linkage_limit=self.linkage_limit,
+                                          min_null_count=self.min_null_count,
+                                          max_null_count=self.max_null_count,
+                                          null_block=self.null_block,
+                                          islands_ok=self.islands_ok,
+                                          short_length=self.short_length,
+                                          display_walls=self.display_walls,
+                                          allow_null=self.allow_null,
+                                          screen_width=self.screen_width,
+                                          max_parse_time=self.max_parse_time,
+                                          max_memory=self.max_memory,
+                                          disjunct_cost=self.disjunct_cost,
+                                          max_sentence_length=self.max_sentence_length)
+        
+        
     def parse_sent(self, txt):
         sent = clg.sentence_create(txt,self.dictionary._dict)
         n = clg.sentence_parse(sent, self.parse_options._po)
+        #This is ugly, but the parser sometiems starts failing on parses.
+        #if it fails, recreate parse_options one time and try again. This will correct the issue.
+        #If the sentence truly cannot be parsed, then it'll fail on the second time.
+        if n == 0:
+            self._create_parse_options()
+            n = clg.sentence_parse(sent, self.parse_options._po)
         if n > 0:
             linkages = [Linkage.create_from_sentence(i,sent, self.parse_options._po) for i in range(n)]
         else:
@@ -46,8 +75,14 @@ class Parser(object):
         #linkgrammar will start to poop its pants if its 'resources' are exhausted.
         #This is a little ugly to check each time but it seems to be a cheap operation.
         #If the resources are exhausted, then reset them.
+        #Extra note: apparently resetting them was not enough. If you kept parsing
+        #the same sentence over and over, eventually the parser were stop returning
+        #parses. I found that deleting and reinstantiating the parse_options corrected
+        #this issue.
         if clg.parse_options_resources_exhausted(self.parse_options._po):
             clg.parse_options_reset_resources(self.parse_options._po)
+            del self.parse_options
+            self._create_parse_options()
         return linkages
     
     def __del__(self):
@@ -90,7 +125,8 @@ class ParseOptions(object):
         self.max_parse_time = max_parse_time
         self.max_memory = max_memory
         self.disjunct_cost = disjunct_cost
-        self.max_sentence_length
+        self.max_sentence_length = max_sentence_length
+        self.allow_null = allow_null
     
     def __del__(self):
         if self._po is not None:
